#region  == Copyright == 
// =====================================================================
//  Microsoft Consulting Services France - Aymeric Mouill - 2013
//  Projet     : TFS WorkItemTracker - TFS WorkItemTracker
//  Fichier    : ChangeSetHistoryForm.cs (01/05/2013 18:40)
//  
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// =====================================================================
#endregion

namespace TFS.WorkItemTracker
{
    #region  == Using == 
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using Microsoft.TeamFoundation.Client;
    using Microsoft.TeamFoundation.VersionControl.Client;
    using Microsoft.TeamFoundation.WorkItemTracking.Client;

    using TFS.WorkItemTracker.Properties;

    #endregion

    public partial class ChangeSetHistoryForm : Form
    {
        private List<ChangeSetLine> ChangeSetLines;

        private readonly WorkItem CurrentWorkItem;

        private int SelectedChangeSetID;

        private VersionControlServer VersionControl;

        #region = Constructor =
        /// <summary>
        /// Initializes a new instance of the <see cref="ChangeSetHistoryForm" /> class.
        /// </summary>
        public ChangeSetHistoryForm(WorkItem workItem)
        {
            CurrentWorkItem = workItem;
            InitializeComponent();
        }
        #endregion

        #region Get Date Version Spec
        /// <summary>
        /// Gets the date version spec.
        /// </summary>
        /// <param name="date">The date.</param>
        /// <returns></returns>
        private static VersionSpec GetDateVersionSpec(DateTime date)
        {
            string dateSpec = string.Format("D{0:yyy}-{0:MM}-{0:dd}T{0:HH}:{0:mm}", date);
            return VersionSpec.ParseSingleSpec(dateSpec, string.Empty);
        }
        #endregion

        #region Change Set History Form Load
        /// <summary>
        /// Handles the Load event of the ChangeSetHistoryForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void ChangeSetHistoryForm_Load(object sender, EventArgs e)
        {
            lblWorkItemInfos.Text = string.Empty;
            lblWorkItemInfos.Text += string.Format("WorkItem ID : {0} | ", CurrentWorkItem.Id);
            lblWorkItemInfos.Text += string.Format("Title : '{0}'\n", CurrentWorkItem.Title);
            lblWorkItemInfos.Text += string.Format("State : {0} | ", CurrentWorkItem.State);
            lblWorkItemInfos.Text += string.Format("Created : {0} | ", CurrentWorkItem.CreatedDate);
            if (CurrentWorkItem.State == "Closed")
            {
                lblWorkItemInfos.Text += string.Format("Closed : {0} | ", CurrentWorkItem.ChangedDate);
            }
            else
            {
                lblWorkItemInfos.Text += string.Format("Changed : {0}", CurrentWorkItem.ChangedDate);
            }

            var dateVersionSpecFrom = GetDateVersionSpec(DateTime.Now - TimeSpan.FromDays(Properties.Settings.Default.HistoryDeltaDays));
            var dateVersionSpecTo = GetDateVersionSpec(DateTime.Now);
            if (CurrentWorkItem.State == "Closed")
            {
                dateVersionSpecFrom = GetDateVersionSpec(CurrentWorkItem.ChangedDate.AddDays(0 - Properties.Settings.Default.HistoryDeltaDays));
                dateVersionSpecTo = GetDateVersionSpec(CurrentWorkItem.ChangedDate.AddDays(Properties.Settings.Default.HistoryDeltaDays));
            }

            TfsTeamProjectCollection tpc = new TfsTeamProjectCollection(new Uri(Properties.Settings.Default.TFSProjectCollectionUrl));

            VersionControl = tpc.GetService<VersionControlServer>();
            var history = VersionControl.QueryHistory("$/CRM Groupe/", // Source control path to the item. Like $/Project/Path ...
                LatestVersionSpec.Instance, // Search latest version
                0, // No unique deletion id. 
                RecursionType.Full, // Full recursion on the path
                null, // All users 
                dateVersionSpecFrom, // From the 14 days ago ...
                dateVersionSpecTo, // LatestVersionSpec.Instance, //To the current version ... 
                Int32.MaxValue, // Include all changes. Can limit the number you get back.
                true, // include details of items. 
                false // Slot mode is false. 
                );

            ChangeSetLines = new List<ChangeSetLine>();
            foreach (Changeset changeset in history)
            {
                var changeSetLine = new ChangeSetLine { Id = changeset.ChangesetId, Owner = changeset.Owner, Date = changeset.CreationDate.ToString(Application.CurrentCulture), Comment = changeset.Comment, Override = changeset.PolicyOverride.Comment };
                foreach (var workItem in changeset.WorkItems)
                {
                    changeSetLine.AssociatedWorkItems += string.Format("{0}; ", workItem.Id);
                }

                ChangeSetLines.Add(changeSetLine);
            }
            gdvChangeSet.DataSource = ChangeSetLines;
        }
        #endregion

        #region Gdv Change Set Cell Click
        /// <summary>
        /// Handles the CellClick event of the gdvChangeSet control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="DataGridViewCellEventArgs" /> instance containing the event data.
        /// </param>
        private void gdvChangeSet_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                SelectedChangeSetID = ChangeSetLines[e.RowIndex].Id;
                txtChangeSetID.Text = SelectedChangeSetID.ToString();
            }
        }
        #endregion

        #region Btn Save Click
        /// <summary>
        /// Handles the Click event of the btnSave control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            var changeSet = VersionControl.GetChangeset(SelectedChangeSetID);

            var workItemStore = new WorkItemStore(Properties.Settings.Default.TFSProjectCollectionUrl);
            RegisteredLinkType type = workItemStore.RegisteredLinkTypes["Fixed in Changeset"];
            var externalLink = new ExternalLink(type, changeSet.ArtifactUri.ToString());

            CurrentWorkItem.Links.Add(externalLink);
            CurrentWorkItem.Save();

            MessageBox.Show(Resources.ChangeSetHistoryForm_btnSave_Click_Operation_completed, Resources.ChangeSetHistoryForm_btnSave_Click_WorkItem_ChangeSet_Association, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        #endregion

        #region Btn Close Click
        /// <summary>
        /// Handles the Click event of the btnClose control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }
        #endregion
    }
}
