#region  == Copyright ==
// =====================================================================
//  Microsoft Consulting Services France - Aymeric Mouill - 2013
//  Projet     : TFS WorkItemTracker - TFS WorkItemTracker
//  Fichier    : WorkItemDetailControl.cs (01/05/2013 18:40)
//  
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// =====================================================================
#endregion

namespace TFS.WorkItemTracker
{
    #region  == Using ==
    using System;
    using System.Windows.Forms;

    using Microsoft.TeamFoundation.WorkItemTracking.Client;

    using TFS.WorkItemTracker.Properties;

    #endregion

    public partial class WorkItemDetailControl : UserControl
    {
        private int CompletedTime;

        private float CompletedTimeHourValue;

        private Timer CompletedTimer;

        private WorkItem CurrentWorkItem;

        private Boolean IsDirty;

        private MainForm MainForm;

        private float OriginalEstimateTime;

        private float RemainingTimeHourValue;

        private bool IsProgressCancelled;

        private const int HOUR_SECONDS = 3600;

        private const int TIMER_INTERVAL = 1000; // 1s

        #region = Constructor =
        /// <summary>
        /// Initializes a new instance of the <see cref="WorkItemDetailControl" /> class.
        /// </summary>
        public WorkItemDetailControl()
        {
            InitializeComponent();
        }
        #endregion

        #region Work Item Detail Control Enter
        /// <summary>
        /// Handles the Enter event of the WorkItemDetailControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void WorkItemDetailControl_Enter(object sender, EventArgs e)
        {
            LoadData();
            IsProgressCancelled = false;
            IsDirty = false;
        }
        #endregion

        #region Work Item Detail Control Visible Changed
        /// <summary>
        /// Handles the VisibleChanged event of the WorkItemDetailControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void WorkItemDetailControl_VisibleChanged(object sender, EventArgs e)
        {
            LoadData();
        }
        #endregion

        #region Load Data
        /// <summary>
        /// Loads the data.
        /// </summary>
        private void LoadData()
        {
            if (string.IsNullOrEmpty(Settings.Default.TFSProjectCollectionUrl))
            {
                return;
            }
            try
            {
                var form = ParentForm as MainForm;
                if (form != null)
                {
                    MainForm = form;
                }
                else
                {
                    return;
                }

                InitializeUI();

                var workItemStore = new WorkItemStore(Settings.Default.TFSProjectCollectionUrl);

                CurrentWorkItem = workItemStore.GetWorkItem(MainForm.SelectedWorkItemId);
                MainForm.SetStatus(string.Format("WorkItem {0} loaded !", CurrentWorkItem.Id));
                propertyGrid.SelectedObject = CurrentWorkItem;

                btnStart.Visible = CurrentWorkItem.State != "Closed";
                btnSave.Visible = CurrentWorkItem.State != "Closed";
                btnAssociate.Visible = CurrentWorkItem.State == "Closed";

                txtTitle.Text = CurrentWorkItem.Title;
                txtDescription.Text = CurrentWorkItem.Description;
                var estimated = Utils.GetFieldValue(CurrentWorkItem, "Original Estimate");
                var completed = Utils.GetFieldValue(CurrentWorkItem, "Completed Work");

                CompletedTime = (int)(float.Parse(completed) * HOUR_SECONDS);

                txtChangeSets.Clear();
                if (CurrentWorkItem.ExternalLinkCount > 0)
                {
                    foreach (var link in CurrentWorkItem.Links)
                    {
                        if (link is ExternalLink)
                        {
                            var changeSet = ((ExternalLink)link).LinkedArtifactUri.Replace("vstfs:///VersionControl/Changeset/", String.Empty);
                            if (!String.IsNullOrEmpty(txtChangeSets.Text))
                            {
                                txtChangeSets.Text += @"; ";
                            }
                            txtChangeSets.Text += changeSet;
                        }
                    }
                }

                OriginalEstimateTime = float.Parse(estimated);
                UpdateProgress();
                Application.DoEvents();

            }
            catch (Exception ex)
            {
                if (MessageBox.Show(ex.ToString(), "An error occured", MessageBoxButtons.RetryCancel, MessageBoxIcon.Stop) == DialogResult.Retry)
                {
                    LoadData();
                }
            }
        }
        #endregion

        #region Initialize UI
        /// <summary>
        /// Initializes the UI.
        /// </summary>
        private void InitializeUI()
        {
            btnStart.Text = Resources.WorkItemDetailControl_InitializeUI_Start;

            txtTitle.Clear();
            txtDescription.Clear();
            txtChangeSets.Clear();

            lblProgress.Text = String.Empty;
            lblProgressPercentage.Text = String.Empty;

            progressBar.Minimum = 0;
            progressBar.Maximum = 100;
            progressBar.Value = 0;

            CompletedTime = 0;
            CompletedTimeHourValue = 0;
            OriginalEstimateTime = 0;
            RemainingTimeHourValue = 0;
        }
        #endregion

        #region Btn Associate Click
        /// <summary>
        /// Handles the Click event of the btnAssociate control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void btnAssociate_Click(object sender, EventArgs e)
        {
            btnAssociate.Enabled = false;
            var form = MainForm.OpenChangeSetHistory(CurrentWorkItem);
            form.FormClosing += form_FormClosing;
        }
        #endregion

        #region Form Form Closing
        private void form_FormClosing(object sender, FormClosingEventArgs e)
        {
            btnAssociate.Enabled = true;
            LoadData();
        }
        #endregion

        #region Btn Return Click
        /// <summary>
        /// Handles the Click event of the btnReturn control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void btnReturn_Click(object sender, EventArgs e)
        {
            MainForm.SwitchToWorkItemList();
        }
        #endregion

        #region Btn Save Click
        /// <summary>
        /// Handles the Click event of the btnSave control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            Save();
            MessageBox.Show(Resources.WorkItemDetailControl_btnSave_Click_Operation_completed, Resources.WorkItemDetailControl_btnSave_Click_WorkItem_Save_Operation, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        #endregion

        #region Btn Start Click
        /// <summary>
        /// Handles the Click event of the btnStart control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (btnStart.Text == Resources.WorkItemDetailControl_InitializeUI_Start)
            {
                IsProgressCancelled = false;
                int percent = (int)(CompletedTimeHourValue / OriginalEstimateTime);
                if (percent >= 0)
                {
                    progressBar.Value = percent;
                }
                CompletedTimer = new Timer();
                CompletedTimer.Interval = TIMER_INTERVAL;
                CompletedTimer.Tick += completedTimer_Tick;
                CompletedTimer.Start();

                btnStart.Text = Resources.WorkItemDetailControl_btnStart_Click_Stop;
                btnAssociate.Enabled = false;
            }
            else
            {
                IsProgressCancelled = true;
                CompletedTimer.Stop();
                CompletedTimer.Tick -= completedTimer_Tick;

                btnStart.Text = Resources.WorkItemDetailControl_InitializeUI_Start;
                btnAssociate.Enabled = true;
            }
        }
        #endregion

        #region Completed Timer Tick
        /// <summary>
        /// Handles the Tick event of the completedTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void completedTimer_Tick(object sender, EventArgs e)
        {
            if (IsProgressCancelled)
            {
                return;
            }
            IsDirty = true;

            CompletedTime++;
            UpdateProgress();

            if (CompletedTime > 0
                && ((CompletedTime % 60) == 0))
            {
#if DEBUG
#else
                Save();
#endif
            }
        }
        #endregion

        #region Update Progress
        /// <summary>
        /// Updates the progress.
        /// </summary>
        private void UpdateProgress()
        {
            if (IsProgressCancelled)
            {
                return;
            }
            CompletedTimeHourValue = CompletedTime / (float)HOUR_SECONDS;
            RemainingTimeHourValue = OriginalEstimateTime - CompletedTimeHourValue;
            if (RemainingTimeHourValue < 0)
            {
                RemainingTimeHourValue = 0;
            }

            lblProgress.Text = string.Format("Estimated : {0} - completed : {1} - remaining : {2}", Utils.ConvertHourValueToHourTimeDisplay(OriginalEstimateTime), Utils.ConvertHourValueToHourTimeDisplay(CompletedTimeHourValue), Utils.ConvertHourValueToHourTimeDisplay(RemainingTimeHourValue));

            if (CompletedTimeHourValue <= OriginalEstimateTime)
            {
                int percent = (int)((CompletedTimeHourValue / OriginalEstimateTime) * 100);
                if (percent >= 0)
                {
                    progressBar.Value = percent;
                    lblProgressPercentage.Text = string.Format("{0}%", percent);
                }
            }

            Application.DoEvents();
        }
        #endregion

        #region Save
        /// <summary>
        /// Saves this instance.
        /// </summary>
        private void Save()
        {
            if (CurrentWorkItem.Type.Name != "Bug")
            {
                CurrentWorkItem["Remaining Work"] = RemainingTimeHourValue.ToString("0.000");
                CurrentWorkItem["Completed Work"] = CompletedTimeHourValue.ToString("0.000");
            }
            CurrentWorkItem.Save();
            IsDirty = false;
        }
        #endregion

        #region Work Item Detail Control Leave
        /// <summary>
        /// Handles the Leave event of the WorkItemDetailControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void WorkItemDetailControl_Leave(object sender, EventArgs e)
        {
            if (IsDirty)
            {
                var diagResult = MessageBox.Show(Resources.WorkItemDetailControl_WorkItemDetailControl_Leave_WorkItem_is_dirty__save, Resources.WorkItemDetailControl_WorkItemDetailControl_Leave_Save, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (diagResult == DialogResult.Yes)
                {
                    Save();
                }
            }

            InitializeUI();

            if (CompletedTimer != null)
            {
                CompletedTimer.Stop();
                CompletedTimer.Dispose();
                CompletedTimer = null;
            }
            CurrentWorkItem = null;
            propertyGrid.SelectedObject = null;
        }
        #endregion
    }
}
