#region  == Copyright ==
// =====================================================================
//  Microsoft Consulting Services France - Aymeric Mouill - 2013
//  Projet     : TFS WorkItemTracker - TFS WorkItemTracker
//  Fichier    : WorkItemsControl.cs (01/05/2013 18:40)
//  
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// =====================================================================
#endregion

namespace TFS.WorkItemTracker
{
    #region  == Using ==
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Windows.Forms;

    using Microsoft.TeamFoundation.WorkItemTracking.Client;

    using TFS.WorkItemTracker.Properties;

    #endregion

    public partial class WorkItemsControl : UserControl
    {
        private String AreaFilterValue;

        private readonly List<String> AreaList;

        private bool IsComboLoaded;

        private String IterationFilterValue;

        private readonly List<String> IterationList;

        private MainForm MainForm;

        private String StateFilterValue;

        private WorkItemStore WorkItemStore;

        #region = Constructor =
        /// <summary>
        /// Initializes a new instance of the <see cref="WorkItemsControl" /> class.
        /// </summary>
        public WorkItemsControl()
        {
            InitializeComponent();
            backgroundWorker.DoWork += backgroundWorker_DoWork;
            backgroundWorker.RunWorkerCompleted += backgroundWorker_RunWorkerCompleted;

            AreaList = new List<String> { string.Empty };
            IterationList = new List<String> { string.Empty };
        }
        #endregion

        #region Work Items Control Load
        /// <summary>
        /// Handles the Load event of the WorkItemsControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void WorkItemsControl_Load(object sender, EventArgs e)
        {
            BindData();
        }
        #endregion

        #region Bind Data
        /// <summary>
        /// Binds the data.
        /// </summary>
        private void BindData()
        {
            if (string.IsNullOrEmpty(Settings.Default.TFSProjectCollectionUrl))
            {
                return;
            }
            try
            {
                MainForm = (MainForm)ParentForm;
                if (MainForm == null)
                {
                    return;
                }
                WorkItemStore = new WorkItemStore(Properties.Settings.Default.TFSProjectCollectionUrl);
                if (!backgroundWorker.IsBusy)
                {
                    backgroundWorker.RunWorkerAsync();
                }
            }
            catch (Exception ex)
            {
                if (MessageBox.Show(ex.ToString(), "An error occured", MessageBoxButtons.RetryCancel, MessageBoxIcon.Stop) == DialogResult.Retry)
                {
                    BindData();
                }
            }
        }
        #endregion

        #region Background Worker Run Worker Completed
        /// <summary>
        /// Handles the RunWorkerCompleted event of the backgroundWorker control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="System.ComponentModel.RunWorkerCompletedEventArgs" /> instance containing the event data.
        /// </param>
        private void backgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            if (MainForm != null
                && e.Result != null)
            {
                var workItemLines = e.Result as List<WorkItemLine>;
                var firstItem = workItemLines.FirstOrDefault();
                if (firstItem != null)
                {
                    MainForm.SelectedWorkItemId = firstItem.Id;
                }
                else
                {
                    MainForm.HideWorkItemDetail();
                }

                MainForm.SetStatus(string.Format("Query results : {0} WorkItem(s) found", workItemLines.Count));

                InitializeComboBox();
                gdvWorkItems.DataSource = null;
                gdvWorkItems.DataSource = workItemLines;
            }
        }
        #endregion

        #region Background Worker Do Work
        /// <summary>
        /// Handles the DoWork event of the backgroundWorker control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="System.ComponentModel.DoWorkEventArgs" /> instance containing the event data.
        /// </param>
        private void backgroundWorker_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            try
            {
                var workItemLines = new List<WorkItemLine>();

                string wiqlQuery = string.Empty;
                wiqlQuery += "SELECT ID, Title ";
                wiqlQuery += "FROM workitems ";
                if (!String.IsNullOrEmpty(StateFilterValue))
                {
                    wiqlQuery += string.Format("WHERE State = '{0}' ", StateFilterValue);
                }
                else
                {
                    wiqlQuery += "WHERE State = 'Active' ";
                }
                wiqlQuery += string.Format("AND [System.AssignedTo] = '{0}' ", WorkItemStore.TeamFoundationServer.AuthenticatedUserDisplayName);
                if (!String.IsNullOrEmpty(IterationFilterValue))
                {
                    wiqlQuery += string.Format("AND [System.IterationPath] UNDER '{0}' ", IterationFilterValue);
                }
                if (!String.IsNullOrEmpty(AreaFilterValue))
                {
                    wiqlQuery += string.Format("AND [System.AreaPath] = '{0}' ", AreaFilterValue);
                }
                wiqlQuery += "ORDER by Title";

                WorkItemCollection witCollection = WorkItemStore.Query(wiqlQuery);
                foreach (WorkItem workItem in witCollection)
                {
                    WorkItemLine workItemLine = Utils.ConvertWorkItemToLine(workItem);
                    workItemLines.Add(workItemLine);

                    if (!IsComboLoaded)
                    {
                        if (!AreaList.Contains(workItem.AreaPath))
                        {
                            AreaList.Add(workItem.AreaPath);
                        }
                        if (!IterationList.Contains(workItem.IterationPath))
                        {
                            IterationList.Add(workItem.IterationPath);
                        }
                    }
                }
                e.Result = workItemLines;
            }
            catch (Exception ex)
            {
                if (MessageBox.Show(ex.ToString(), "An error occured", MessageBoxButtons.RetryCancel, MessageBoxIcon.Stop) == DialogResult.Retry)
                {
                    BindData();
                }
            }
        }
        #endregion

        #region Initialize Combo Box
        /// <summary>
        /// Initializes the combo box.
        /// </summary>
        private void InitializeComboBox()
        {
            if (!IsComboLoaded)
            {
                AreaList.Sort();
                cbxArea.DataSource = AreaList;

                IterationList.Sort();
                cbxIteration.DataSource = IterationList;

                IsComboLoaded = true;
            }
        }
        #endregion

        #region Cbx Area Selected Index Changed
        /// <summary>
        /// Handles the SelectedIndexChanged event of the cbxArea control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void cbxArea_SelectedIndexChanged(object sender, EventArgs e)
        {
            AreaFilterValue = cbxArea.SelectedItem.ToString();
            if (!backgroundWorker.IsBusy)
            {
                backgroundWorker.RunWorkerAsync();
            }
        }
        #endregion

        #region Cbx Iteration Selected Index Changed
        /// <summary>
        /// Handles the SelectedIndexChanged event of the cbxIteration control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void cbxIteration_SelectedIndexChanged(object sender, EventArgs e)
        {
            IterationFilterValue = cbxIteration.SelectedItem.ToString();
            if (!backgroundWorker.IsBusy)
            {
                backgroundWorker.RunWorkerAsync();
            }
        }
        #endregion

        #region Gdv Work Items Cell Click
        /// <summary>
        /// Handles the CellClick event of the gdvWorkItems control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="DataGridViewCellEventArgs" /> instance containing the event data.
        /// </param>
        private void gdvWorkItems_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                var cellId = (int)gdvWorkItems[0, e.RowIndex].Value;

                MainForm.SelectedWorkItemId = cellId;
                MainForm.SwitchToWorkItemDetail();
            }
        }
        #endregion

        #region Cbx State Selected Index Changed
        /// <summary>
        /// Handles the SelectedIndexChanged event of the cbxState control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void cbxState_SelectedIndexChanged(object sender, EventArgs e)
        {
            StateFilterValue = cbxState.SelectedItem.ToString();
            if (!backgroundWorker.IsBusy)
            {
                backgroundWorker.RunWorkerAsync();
            }
        }
        #endregion

        #region Work Items Control Visible Changed
        /// <summary>
        /// Handles the VisibleChanged event of the WorkItemsControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">
        /// The <see cref="EventArgs" /> instance containing the event data.
        /// </param>
        private void WorkItemsControl_VisibleChanged(object sender, EventArgs e)
        {
            if (!backgroundWorker.IsBusy)
            {
                backgroundWorker.RunWorkerAsync();
            }
        }
        #endregion
    }
}
